import React, { PropsWithChildren } from 'react';
import { Button } from '../buttons';
import { Close } from '@mui/icons-material';

export const FormDiv = React.forwardRef<HTMLDivElement, PropsWithChildren<{ className?: string; hAnchor?: string; offset?: string }>>(
  (props, ref) => {
    const dialogRef = React.useRef<HTMLDivElement | null>(null);
    const isClicked = React.useRef<boolean>(false);
    const initialPosition = React.useRef<{ x: number; y: number }>({ x: 0, y: 0 });

    React.useEffect(() => {
      if (!dialogRef.current) return;

      const dialog = dialogRef.current;
      const container = document.body;

      const onMouseDown = (e: MouseEvent) => {
        isClicked.current = true;
        initialPosition.current = {
          x: e.clientX - dialog.getBoundingClientRect().left,
          y: e.clientY - dialog.getBoundingClientRect().top,
        };
      };

      const onMouseUp = () => {
        isClicked.current = false;
      };

      const onMouseMove = (e: MouseEvent) => {
        if (!isClicked.current) return;
        const newX = e.clientX - initialPosition.current.x;
        const newY = e.clientY - initialPosition.current.y;
        dialog.style.top = `${newY}px`;
        dialog.style.left = `${newX}px`;
      };

      dialog.addEventListener('mousedown', onMouseDown);
      dialog.addEventListener('mouseup', onMouseUp);
      container.addEventListener('mousemove', onMouseMove);
      container.addEventListener('mouseleave', onMouseUp);

      return () => {
        dialog.removeEventListener('mousedown', onMouseDown);
        dialog.removeEventListener('mouseup', onMouseUp);
        container.removeEventListener('mousemove', onMouseMove);
        container.removeEventListener('mouseleave', onMouseUp);
      };
    }, []);

    return (
      <div
        className={
          'absolute opacity-100 transition-opacity group-hover:opacity-100 z-50 w-fit cursor-move' +
          (props.className ? props.className : '')
        }
        ref={(node) => {
          dialogRef.current = node;
          if (typeof ref === 'function') {
            ref(node);
          } else if (ref) {
            ref.current = node;
          }
        }}
        style={props.hAnchor === 'left' ? { left: props.offset || 0 } : { right: props.offset || '5rem' }}
      >
        {props.children}
      </div>
    );
  },
);
export const FormCard = (props: PropsWithChildren<{ className?: string }>) => (
  <div className={'card card-bordered bg-light rounded-none ' + (props.className ? props.className : '')}>{props.children}</div>
);
export const FormBody = ({
  children,
  ...props
}: PropsWithChildren<React.DetailedHTMLProps<React.FormHTMLAttributes<HTMLFormElement>, HTMLFormElement>>) => (
  <form className="card-body px-0 w-72 py-0" {...props}>
    {children}
  </form>
);
export const FormTitle = ({ children, title, onClose }: PropsWithChildren<{ title: string; onClose: () => void }>) => {
  return (
    <div className="card-title p-5 py-0 mt-2 flex w-full">
      <h2 className="w-full">{title}</h2>
      <Button rounded variant="ghost" iconComponent={<Close />} onClick={() => onClose()} />
    </div>
  );
};
export const FormHBar = () => <div className="divider m-0"></div>;
export const FormControl = ({ children }: PropsWithChildren) => <div className="form-control px-5">{children}</div>;
export const FormActions = (props: { onClose: () => void }) => (
  <div className="grid grid-cols-2 px-5 gap-2 mb-2">
    <Button
      type="secondary"
      variant="outline"
      label="Cancel"
      onClick={(e) => {
        e.preventDefault();
        props.onClose();
      }}
    />
    <Button type="primary" label="Apply" onClick={() => {}} />
  </div>
);
