import React, { useState, useEffect, Suspense } from 'react';
import { VISComponentType, VisualizationConfiguration } from '../types';
import {
  removeVisualization,
  reorderVisState,
  setActiveVisualization,
  updateVisualization,
} from '../../data-access/store/visualizationSlice';
import {
  useAppDispatch,
  useGraphQueryResult,
  useGraphQueryResultMeta,
  useML,
  useSchemaGraph,
  useSessionCache,
  useVisualization,
} from '../../data-access';
import { VisualizationManager } from './manager.types';

export const Visualizations: Record<string, Function> = {
  TableVis: () => import('../visualizations/tablevis/tablevis'),
  PaohVis: () => import('../visualizations/paohvis/paohvis'),
  RawJSONVis: () => import('../visualizations/rawjsonvis/rawjsonvis'),
  NodeLinkVis: () => import('../visualizations/nodelinkvis/nodelinkvis'),
  // MapVis: () => import('./visualizations/mapvis/mapvis'),
  MatrixVis: () => import('../visualizations/matrixvis/matrixvis'),
};

export const VISUALIZATION_TYPES: string[] = Object.keys(Visualizations);

export const useVisualizationManager = (): VisualizationManager => {
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const ml = useML();
  const schema = useSchemaGraph();
  const graphQueryResult = useGraphQueryResult();
  const meta = useGraphQueryResultMeta();
  const { active, visualizations } = useVisualization();

  const [configuration, setConfiguration] = useState<any>();
  const [visualization, setVisualization] = useState<VISComponentType>();
  const [hoverItem, setHoverItem] = useState<any>();
  const [selected, setSelected] = useState<any>();
  const activeType = active ? visualizations[active]?.displayName : undefined;
  const tabs = Object.keys(visualizations).length ? Object.keys(visualizations) : [];

  useEffect(() => {
    console.log('use effect called');
    loadVisualization();
  }, [active]);

  const loadVisualization = async () => {
    console.log('loading vis');
    if (active && Visualizations[active]) {
      const componentModule = await Visualizations[active]();
      const component = componentModule.default;

      if (!(active in Object.keys(visualizations))) {
        console.log('Not in redux yet');
        // Visualization doesn't yet exist so add its configuration
        const configuration = component.configuration;
        dispatch(updateVisualization({ id: active, settings: configuration }));
        setConfiguration(configuration);
      } else {
        setConfiguration(visualizations[active]);
      }

      setVisualization(component);
    }
  };

  const changeActive = (id: string) => {
    console.log('CHANGING VIS TO', id);
    dispatch(setActiveVisualization(id));
    // loadVisualization(id);
  };

  const deleteVisualization = (id: string) => {
    dispatch(removeVisualization(id));
    if (Object.keys(visualizations).length > 0) {
      const newActive = tabs.find((v: string) => id !== v);
      changeActive(newActive || '');
    }
  };

  const reorderVisualizations = ({ draggedVisId, dropVisId }: { draggedVisId: string; dropVisId: string }) => {
    const settingsCopy = { ...visualizations };
    const keys = Object.keys(settingsCopy);
    const draggedIndex = keys.indexOf(draggedVisId);
    const dropIndex = keys.indexOf(dropVisId);

    if (draggedIndex !== -1 && dropIndex !== -1) {
      keys.splice(dropIndex, 0, draggedVisId);
      const newSettings: { [id: string]: VisualizationConfiguration } = {};
      keys.forEach((key) => {
        newSettings[key] = settingsCopy[key];
      });

      dispatch(reorderVisState(newSettings));
    }
  };

  const handleHover = (item: any) => {
    setHoverItem(item);
  };

  const handleSelect = (item: any) => {
    setSelected(item);
  };

  const updateSettings = (newSettings: any) => {
    if (active) {
      console.log('updating settings');
      const updatedSettings = { ...configuration, ...newSettings };
      setConfiguration(updatedSettings);
      dispatch(updateVisualization({ id: active, settings: updatedSettings }));
    }
  };

  const renderSettings = () => {
    return (
      visualization?.settings &&
      configuration && <visualization.settings configuration={configuration} graph={meta} updateSettings={updateSettings} />
    );
  };

  const renderComponent = () => {
    return (
      <Suspense fallback={<div>Loading...</div>}>
        {visualization?.component && configuration && (
          <visualization.component
            data={graphQueryResult}
            schema={schema}
            ml={ml}
            configuration={configuration}
            dispatch={dispatch}
            handleHover={handleHover}
            handleSelect={handleSelect}
          />
        )}
      </Suspense>
    );
  };

  return {
    renderComponent,
    renderSettings,
    active,
    activeType,
    tabs,
    changeActive,
    reorderVisualizations,
    deleteVisualization,
  };
};
